"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TaskOperationService = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@loopback/core");
const repository_1 = require("@loopback/repository");
const camunda_service_1 = require("./camunda.service");
const bpmn_service_1 = require("@sourceloop/bpmn-service");
const commands_1 = require("../commands");
const keys_1 = require("../keys");
const types_1 = require("../types");
const workflow_operation_service_1 = require("./workflow-operation.service");
const utility_service_1 = require("./utility.service");
const task_db_service_1 = require("./task-db.service");
const rest_1 = require("@loopback/rest");
const webhook_service_1 = require("./webhook.service");
const repositories_1 = require("../repositories");
let TaskOperationService = class TaskOperationService {
    constructor(taskDbService, camundaService, workflowOpsService, utilityService, regFn, workerFn, customBpmnRunner, webhookService, taskRepo) {
        this.taskDbService = taskDbService;
        this.camundaService = camundaService;
        this.workflowOpsService = workflowOpsService;
        this.utilityService = utilityService;
        this.regFn = regFn;
        this.workerFn = workerFn;
        this.customBpmnRunner = customBpmnRunner;
        this.webhookService = webhookService;
        this.taskRepo = taskRepo;
        this.clientBpmnRunner = {
            workerFunctions: {},
            tasksArray: [],
        };
        this.clientBpmnRunner = this.customBpmnRunner;
    }
    _createTaskProcessorFunction(updatedPayload, task) {
        return (taskOfWorkerFn, taskServiceOfWorkerFn) => {
            // logic given by the consumer to be plugged in
            return this.clientBpmnRunner.workerFunctions[task.topicName](taskOfWorkerFn, taskServiceOfWorkerFn, updatedPayload);
        };
    }
    async processTask(id, name, key, payload) {
        const tasks = await this.camundaService.getCurrentExternalTask(id);
        if (tasks && tasks.length > 0) {
            const tasksPromises = tasks.map(task => this._registerOrUpdateCommand(task, { key, payload, name, id }));
            await Promise.all(tasksPromises);
        }
        else {
            if (this.clientBpmnRunner.tasksArray.length > 0) {
                const tasksArrayPromises = this.clientBpmnRunner.tasksArray.map(task => this._createTaskAndExecuteWorkflow(task));
                await Promise.all(tasksArrayPromises);
            }
            if (payload) {
                const messageDTO = {
                    message: 'Event Proccessed',
                    key: key,
                    payload,
                    eventKey: key,
                    taskKey: 'none',
                };
                await this.webhookService.triggerWebhook(key, messageDTO);
            }
            this.clientBpmnRunner.tasksArray.length = 0;
        }
        await this._initWorkers(name);
    }
    async taskUpdateFlow(taskKey, payload) {
        try {
            const taskWorkflowMapping = await this.workflowOpsService.findTaskWorkflowByKey(taskKey);
            if (!taskWorkflowMapping)
                throw new rest_1.HttpErrors.NotFound('No workflow mapped to task');
            const workflow = await this.workflowOpsService.findWorkflowByKey(taskWorkflowMapping.workflowKey);
            if (!workflow)
                throw new rest_1.HttpErrors.NotFound('No workflow found with key');
            const userTasks = await this.camundaService.getCurrentUserTask(workflow.externalIdentifier);
            if (!userTasks || userTasks.length === 0)
                return;
            const transformedPayload = this.utilityService.transformObject(payload);
            for (const ut of userTasks) {
                if (!ut.id)
                    continue;
                await this.camundaService.completeUserTask(ut.id, transformedPayload);
                // check if there are Task tasks left
                const pendingTasks = await this.camundaService.getCurrentUserTask(workflow.externalIdentifier);
                ut.status =
                    (pendingTasks === null || pendingTasks === void 0 ? void 0 : pendingTasks.length) > 0
                        ? types_1.TaskStatus.in_progress
                        : types_1.TaskStatus.completed;
                await this.taskDbService.updateTaskStatus(taskKey, ut);
                if (payload) {
                    const messageDTO = {
                        message: 'Event Proccessed',
                        key: taskKey,
                        payload,
                        eventKey: 'none',
                        taskKey: taskKey,
                    };
                    await this.webhookService.triggerWebhook(taskKey, messageDTO);
                }
            }
        }
        catch (error) {
            throw new rest_1.HttpErrors.InternalServerError(`could not update the task`);
        }
    }
    async _registerOrUpdateCommand(task, data) {
        const topic = task.topicName;
        const taskProcessorFn = this._createTaskProcessorFunction(data.payload, task);
        const toStart = await this._runCommand(types_1.TaskServiceNames.CHECK, data.name, topic);
        const cmd = new commands_1.TaskProcessorCommand(taskProcessorFn, this.processTask.bind(this), [data.id, data.name, data.key]);
        if (!toStart) {
            await this.regFn(data.name, topic, new bpmn_service_1.BPMTask(cmd));
        }
        else {
            await this._runCommand(types_1.TaskServiceNames.UPDATE, data.name, task.topicName, new bpmn_service_1.BPMTask(cmd));
        }
    }
    async _runCommand(type, workflowName, topic, cmd) {
        return this.utilityService.iterateWorkerMap(workflowName, async (worker) => {
            if (worker.topic === topic && worker.running) {
                if (type === types_1.TaskServiceNames.UPDATE && cmd) {
                    worker.command = cmd;
                    return true;
                }
                else if (type === types_1.TaskServiceNames.CHECK) {
                    return true;
                }
                else {
                    return false;
                }
            }
            return false;
        });
    }
    async _initWorkers(workflowName) {
        await this.utilityService.iterateWorkerMap(workflowName, async (worker) => {
            if (!worker.running) {
                await this.workerFn(worker);
                worker.running = true;
                return true;
            }
            return false;
        });
    }
    async _createTaskAndExecuteWorkflow(task) {
        await this.taskRepo.create({
            key: task.key,
            name: task.name,
            description: task.description,
            status: task.status,
            severity: task.severity,
            priority: task.priority,
            type: task.type,
            assigneeId: task.assigneeId,
            startDate: task.startDate,
            dueDate: task.dueDate,
            endDate: task.endDate,
        });
        await this.workflowOpsService.execWorkflow(task.key, types_1.TaskServiceNames.TASK);
    }
};
exports.TaskOperationService = TaskOperationService;
exports.TaskOperationService = TaskOperationService = tslib_1.__decorate([
    (0, core_1.injectable)({ scope: core_1.BindingScope.TRANSIENT }),
    tslib_1.__param(0, (0, core_1.service)(task_db_service_1.TaskDbService)),
    tslib_1.__param(1, (0, core_1.service)(camunda_service_1.CamundaService)),
    tslib_1.__param(2, (0, core_1.service)(workflow_operation_service_1.WorkflowOperationService)),
    tslib_1.__param(3, (0, core_1.service)(utility_service_1.UtilityService)),
    tslib_1.__param(4, (0, core_1.inject)(bpmn_service_1.WorkflowServiceBindings.RegisterWorkerFunction)),
    tslib_1.__param(5, (0, core_1.inject)(bpmn_service_1.WorkflowServiceBindings.WorkerImplementationFunction)),
    tslib_1.__param(6, (0, core_1.inject)(keys_1.TaskServiceBindings.CUSTOM_BPMN_RUNNER)),
    tslib_1.__param(7, (0, core_1.service)(webhook_service_1.WebhookService)),
    tslib_1.__param(8, (0, repository_1.repository)(repositories_1.TaskRepository)),
    tslib_1.__metadata("design:paramtypes", [task_db_service_1.TaskDbService,
        camunda_service_1.CamundaService,
        workflow_operation_service_1.WorkflowOperationService,
        utility_service_1.UtilityService, Function, Function, Object, webhook_service_1.WebhookService,
        repositories_1.TaskRepository])
], TaskOperationService);
//# sourceMappingURL=task-operation.service.js.map